from django.db.models.fields.files import FieldFile
from rest_framework.exceptions import ValidationError
from rest_framework.fields import SkipField

from drf_file_upload.fields import UploadedFileField
from tests.tests import factory
from tests.tests.base_test import BaseDrfFileUploadTestCase, FakeRequest, FAKE_REQUEST_DOMAIN


class UploadedFileFieldTestCase(BaseDrfFileUploadTestCase):
    def setUp(self):
        super().setUp()

        self.user = factory.create_user()
        self.sut = UploadedFileField()
        self.sut._context = {
            "request": FakeRequest(self.user)
        }

    def test_to_internal_field_with_valid_uuid_returns_uploaded_file_instance(self):
        file = factory.create_authenticated_uploaded_file(self.user, "nice_pic.png")
        value = self.sut.to_internal_value(file.uuid)
        self.assertIsInstance(value, FieldFile)

    def test_to_internal_field_parameter_must_be_string(self):
        with self.assertRaises(ValidationError) as error:
            self.sut.to_internal_value(22)

        self.assertEquals(error.exception.detail[0].code, "invalid")
        self.assertEquals(str(error.exception.detail[0]), "uploaded-file-invalid-uploaded-file")

    def test_to_internal_field_parameter_must_be_valid(self):
        with self.assertRaises(ValidationError) as error:
            self.sut.to_internal_value("f" * 35)

        self.assertEquals(error.exception.detail[0].code, "invalid")
        self.assertEquals(str(error.exception.detail[0]), "uploaded-file-invalid-uuid")

        with self.assertRaises(ValidationError) as error:
            self.sut.to_internal_value("f" * 37)

        self.assertEquals(error.exception.detail[0].code, "invalid")
        self.assertEquals(str(error.exception.detail[0]), "uploaded-file-invalid-uuid")

    def test_to_internal_field_parameter_must_exist(self):
        with self.assertRaises(ValidationError) as error:
            self.sut.to_internal_value("f" * 36)

        self.assertEquals(error.exception.detail[0].code, "invalid")
        self.assertEquals(str(error.exception.detail[0]), "uploaded-file-uuid-not-found")

    def test_to_internal_skips_if_url_is_given(self):
        with self.assertRaises(SkipField):
            self.sut.to_internal_value("http://drf-upload-file.test/hello.jpg")
        with self.assertRaises(SkipField):
            self.sut.to_internal_value("https://drf-upload-file.test/hello.jpg")

    def test_to_representation(self):
        file = factory.create_authenticated_uploaded_file(self.user, "nice_pic.png")

        repr = self.sut.to_representation(file.file)
        self.assertTrue(repr.startswith(FAKE_REQUEST_DOMAIN))
        self.assertTrue(repr.endswith(".png"))



from django.urls import include, path
from rest_framework import routers

from drf_file_upload import views

router = routers.SimpleRouter()
router.register(r"upload", views.AuthenticatedFileUploadView, basename="authenticated-upload")
router.register(
    r"anonymous-upload",
    views.AnonymousFileUploadView,
    basename="anonymous-upload",
)

urlpatterns = [
    path("", include(router.urls)),
]



from rest_framework import serializers

import drf_file_upload.fields
from drf_file_upload import serializers as dfu_serializers
from tests import models as test_models


class TestUserFileUploadSerializer(dfu_serializers.UploadedFileModelSerializer):

    user = serializers.HiddenField(default=serializers.CurrentUserDefault())
    file = drf_file_upload.fields.UploadedFileField(required=True, allow_null=False)

    class Meta:
        model = test_models.TestUserFileUpload
        fields = ("user", "file", "created")


class TestAnonFileUploadSerializer(dfu_serializers.UploadedFileModelSerializer):

    file = drf_file_upload.fields.AnonymousUploadedFileField(required=True, allow_null=False)

    class Meta:
        model = test_models.TestAnonFileUpload
        fields = ("file", "created")



# Generated by Django 3.2 on 2021-04-17 15:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drf_file_upload', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='authenticateduploadedfile',
            name='uuid',
            field=models.CharField(default=None, max_length=64, null=True, unique=True),
        ),
    ]



from django.core import management

from tests.tests import factory
from tests.tests.base_test import BaseDrfFileUploadTestCase


class TestSpectacularSchemaTestCase(BaseDrfFileUploadTestCase):
    def setUp(self):
        super().setUp()
        self.user = factory.create_user()

    def test_spectacular_schema(self):
        management.call_command("spectacular", file="test_schema.yml")

        # TODO: Validate schema..



import io
from datetime import datetime

import pytz
from PIL import Image
from django.contrib.auth.models import User
from django.core.files import File

from drf_file_upload import models

USERNAME = "test_upload"
PASSWORD = "test_upload!"


def create_simple_uploaded_file(filename="best_file_eva.pdf", size=(100, 100), color=(155, 0, 0)):
    file = io.BytesIO()
    image = Image.new("RGBA", size=size, color=color)
    image.save(file, "png")
    file.name = filename
    file.seek(0)
    return file


def create_user():
    return User.objects.create_user(username=USERNAME, password=PASSWORD)


def create_datetime(year, month, day, hour, minutes=0, seconds=0):
    mock_date = datetime(year, month, day, hour, minutes, seconds)
    tz = pytz.timezone("Europe/Rome")
    mock_date = tz.localize(mock_date)
    return mock_date


def create_authenticated_uploaded_file(user, filename="test_file.pdf", size=(100, 100)):
    file_contents = create_simple_uploaded_file(filename=filename, size=size)
    return models.AuthenticatedUploadedFile.objects.create(file=File(file_contents), user=user)


def create_anonymous_uploaded_file(filename="test_file.pdf", size=(100, 100)):
    file_contents = create_simple_uploaded_file(filename=filename, size=size)
    return models.AnonymousUploadedFile.objects.create(file=File(file_contents))



from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = "fake-key"


INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "drf_spectacular",
    "drf_file_upload",
    "tests",
]

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

ROOT_URLCONF = "tests.urls"


REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "rest_framework.authentication.BasicAuthentication",
        "rest_framework.authentication.SessionAuthentication",
    ],
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
}

USE_TZ = True

TIME_ZONE = "Europe/Rome"



from rest_framework.test import APIClient

from drf_file_upload import models
from tests import serializers as test_serializers
from tests.tests import factory
from tests.tests.base_test import BaseDrfFileUploadTestCase, FakeRequest


class AuthenticatedFileUploadTestCase(BaseDrfFileUploadTestCase):
    def setUp(self):
        super().setUp()
        self.client = APIClient()
        self.user = factory.create_user()

        self.uploaded_file = factory.create_authenticated_uploaded_file(self.user)

    def test_upload_model_serializer_valid_data(self):
        sut = self.create_serializer(self.uploaded_file.uuid)

        self.assertTrue(sut.is_valid())
        test_file_model = sut.save()

        self.assertIsNotNone(test_file_model, "The test model is returned")
        self.assertIsNotNone(test_file_model.file, "The test model has a file")

        self.assertFalse(
            models.AuthenticatedUploadedFile.objects.filter(id=self.uploaded_file.id).exists(),
            "Uploaded file instance is deleted on save",
        )

    def test_upload_model_serializer_invalid_data(self):
        sut = self.create_serializer("not-a-valid-file-uuid")

        self.assertFalse(sut.is_valid())
        self.assertEqual(len(sut.errors), 1, "There is one validation error")
        self.assertTrue("file" in sut.errors, "The validation error is for the file")
        self.assertEqual(sut.errors["file"][0].code, "invalid")

    def create_serializer(self, file_id):
        sut = test_serializers.TestUserFileUploadSerializer(
            data={"file": file_id}, context={"request": FakeRequest(self.user)}
        )
        return sut



# Generated by Django 3.2 on 2021-04-17 10:03

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import drf_file_upload.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="AnonymousUploadedFile",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("file", models.FileField(upload_to=drf_file_upload.models.get_anonymous_uploaded_file_path)),
                ("uuid", models.CharField(max_length=64, unique=True)),
                ("created", models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name="AuthenticatedUploadedFile",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("file", models.FileField(upload_to=drf_file_upload.models.get_authenticated_uploaded_file_path)),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("user", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import os
from unittest.mock import patch

from django.core import management
from django.utils import timezone

from drf_file_upload import models
from tests.tests import factory
from tests.tests.base_test import BaseDrfFileUploadTestCase


class TestDeleteExpiredUploadedFilesTestCase(BaseDrfFileUploadTestCase):
    def setUp(self):
        super().setUp()
        self.user = factory.create_user()

    def tearDown(self):
        models.AnonymousUploadedFile.objects.all().delete()
        models.AuthenticatedUploadedFile.objects.all().delete()

    def test_delete_expired_uploaded_files_deletes_expired_files(self):
        expired_date = factory.create_datetime(2021, 4, 17, 14, 2, 59)
        with patch.object(timezone, "now", return_value=expired_date):
            auth_exp_1 = factory.create_authenticated_uploaded_file(self.user)
            auth_exp_2 = factory.create_authenticated_uploaded_file(self.user)
            anon_exp_1 = factory.create_anonymous_uploaded_file()

        self.assertTrue(os.path.isfile(auth_exp_1.file.path))
        self.assertTrue(os.path.isfile(auth_exp_2.file.path))
        self.assertTrue(os.path.isfile(anon_exp_1.file.path))

        not_expired_date = factory.create_datetime(2021, 4, 17, 14, 3)
        with patch.object(timezone, "now", return_value=not_expired_date):
            auth_valid_1 = factory.create_authenticated_uploaded_file(self.user)
            auth_valid_2 = factory.create_authenticated_uploaded_file(self.user)
            anon_valid_1 = factory.create_anonymous_uploaded_file()
            anon_valid_2 = factory.create_anonymous_uploaded_file()
            anon_valid_3 = factory.create_anonymous_uploaded_file()

        cleanup_date = factory.create_datetime(2021, 4, 17, 14, 4)
        with patch.object(timezone, "now", return_value=cleanup_date):
            management.call_command("delete_expired_uploaded_files", seconds=60)

        self.assertEqual(models.AuthenticatedUploadedFile.objects.count(), 2)
        self.assertEqual(models.AnonymousUploadedFile.objects.count(), 3)

        for auth_valid in (auth_valid_1, auth_valid_2):
            self.assertTrue(models.AuthenticatedUploadedFile.objects.filter(pk=auth_valid.id).exists())

        for anon_valid in (anon_valid_1, anon_valid_2, anon_valid_3):
            self.assertTrue(models.AnonymousUploadedFile.objects.filter(pk=anon_valid.id).exists())

        self.assertFalse(os.path.exists(auth_exp_1.file.path))
        self.assertFalse(os.path.exists(auth_exp_2.file.path))
        self.assertFalse(os.path.exists(anon_exp_1.file.path))



import mimetypes

from django.conf import settings
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from drf_file_upload import models
from drf_file_upload.fields import AnonymousUploadedFileField, UploadedFileField


class UploadFileValidationMixin:
    def validate_file(self, value):
        if not self.is_supported_file(value.name):
            raise ValidationError("invalid-file-format")
        if not self.respects_filesize_limit(value.size):
            raise ValidationError("file-too-large")
        return value

    def is_supported_file(self, file):
        if not hasattr(settings, "DRF_FILE_UPLOAD_ALLOWED_FORMATS") or not settings.DRF_FILE_UPLOAD_ALLOWED_FORMATS:
            return True

        mimetype = mimetypes.guess_type(file)[0]
        return mimetype in settings.DRF_FILE_UPLOAD_ALLOWED_FORMATS

    def respects_filesize_limit(self, size):
        if not hasattr(settings, "DRF_FILE_UPLOAD_MAX_SIZE") or not settings.DRF_FILE_UPLOAD_MAX_SIZE:
            return True
        return size <= settings.DRF_FILE_UPLOAD_MAX_SIZE


class AuthenticatedUploadFileSerializer(UploadFileValidationMixin, serializers.ModelSerializer):
    user = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = models.AuthenticatedUploadedFile
        fields = ["file", "user", "id"]
        read_only_fields = ["id"]
        # swagger_schema_fields = {
        #     "type": openapi.TYPE_OBJECT,
        #     "title": "Upload file",
        #     "properties": {
        #         "id": openapi.Schema(title="File Id", type=openapi.TYPE_NUMBER, read_only=True),
        #         "file": openapi.Schema(
        #             title="Multipart file",
        #             type=openapi.TYPE_STRING,
        #         ),
        #     },
        # }


class AnonymousUploadFileSerializer(UploadFileValidationMixin, serializers.ModelSerializer):
    class Meta:
        model = models.AnonymousUploadedFile
        fields = ["file", "uuid"]
        read_only_fields = ["uuid"]
        # swagger_schema_fields = {
        #     "type": openapi.TYPE_OBJECT,
        #     "title": "Upload file",
        #     "properties": {
        #         "uuid": openapi.Schema(title="File Unique ID", type=openapi.TYPE_STRING, read_only=True),
        #         "file": openapi.Schema(
        #             title="Multipart file",
        #             type=openapi.TYPE_STRING,
        #         ),
        #     },
        # }


class UploadedFileSerializerMixin:
    def clean_uploaded_files(self):
        for field_name, field_type in self.get_fields().items():
            if isinstance(field_type, AnonymousUploadedFileField) and field_name in self.validated_data:
                models.AnonymousUploadedFile.objects.filter(file=self.validated_data[field_name]).delete()
            if isinstance(field_type, UploadedFileField) and field_name in self.validated_data:
                models.AuthenticatedUploadedFile.objects.filter(file=self.validated_data[field_name]).delete()


class UploadedFileSerializer(UploadedFileSerializerMixin, serializers.Serializer):
    def save(self, **kwargs):
        self.clean_uploaded_files()
        return super().save(**kwargs)


class UploadedFileModelSerializer(UploadedFileSerializerMixin, serializers.ModelSerializer):
    def save(self, **kwargs):
        self.clean_uploaded_files()
        return super().save(**kwargs)



from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.fields import SkipField

from drf_file_upload import models


class AbstractUploadedFileField(serializers.Field):
    requires_context = True

    def to_internal_value(self, data):
        if not isinstance(data, str):
            raise ValidationError("uploaded-file-invalid-uploaded-file")

        is_new_file = not data.startswith("http")

        if is_new_file:
            if len(data) != 36:
                raise ValidationError("uploaded-file-invalid-uuid")

            return self.fetch_file_or_error(data)

        # Storing the value again, without changes: This is expected to be the original URL
        raise SkipField()

    def to_representation(self, value):
        if not value:
            return None

        try:
            url = value.url
        except AttributeError:
            return None
        request = self.context.get("request", None)
        if request is not None:
            return request.build_absolute_uri(url)
        return url

    def fetch_file_or_error(self, uuid):
        raise NotImplementedError("Must implement fetch_file_or_error")


class UploadedFileField(AbstractUploadedFileField):

    def fetch_file_or_error(self, uuid):
        user = self.context["request"].user
        try:
            file = models.AuthenticatedUploadedFile.objects.get(uuid=uuid, user=user)
            return file.file
        except models.AuthenticatedUploadedFile.DoesNotExist:
            raise ValidationError(f"uploaded-file-uuid-not-found")


class AnonymousUploadedFileField(AbstractUploadedFileField):

    def fetch_file_or_error(self, uuid):
        try:
            file = models.AnonymousUploadedFile.objects.get(uuid=uuid)
            return file.file
        except models.AnonymousUploadedFile.DoesNotExist:
            raise ValidationError(f"uploaded-file-uuid-not-found")



from django.conf import settings
from rest_framework import status
from rest_framework.test import APIClient

from drf_file_upload import models
from tests.tests import factory
from tests.tests.base_test import BaseDrfFileUploadTestCase

API_ENDPOINT = "/upload/"


class AuthenticatedFileUploadTestCase(BaseDrfFileUploadTestCase):

    def setUp(self):
        super().setUp()
        self.client = APIClient()
        self.user = factory.create_user()

    def test_file_upload_requires_logged_user(self):
        response = self.upload_file(authenticate=False)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

        self.assertEqual(models.AuthenticatedUploadedFile.objects.count(), 0)

    def test_logged_user_can_upload_file(self):
        response = self.upload_file()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        response_data = response.json()
        self.assertTrue(response_data["id"] > 0)
        self.assertTrue(response_data["file"].startswith("http"))

        auth_uploaded_file = models.AuthenticatedUploadedFile.objects.get(id=response_data["id"])
        self.assertIsNotNone(auth_uploaded_file.file)
        self.assertEqual(auth_uploaded_file.user, self.user)

    def test_upload_file_max_size_returns_error_with_file_too_large(self):
        settings.DRF_FILE_UPLOAD_MAX_SIZE = 100
        response = self.upload_file()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        error = response.json()
        self.assertTrue("file" in error)
        self.assertEqual(error["file"], ["file-too-large"])

    def test_upload_file_max_size(self):
        settings.DRF_FILE_UPLOAD_MAX_SIZE = 100000
        response = self.upload_file()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_upload_file_allowed_type_returns_error_with_invalid_type(self):
        settings.DRF_FILE_UPLOAD_ALLOWED_FORMATS = {"image/png", "image/jpg"}

        response = self.upload_file(filename="invalid.pdf")

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        error = response.json()
        self.assertTrue("file" in error)
        self.assertEqual(error["file"], ["invalid-file-format"])

    def test_upload_file_allowed_type(self):
        settings.DRF_FILE_UPLOAD_ALLOWED_FORMATS = {"image/png", "image/jpg", "application/pdf"}

        response = self.upload_file(filename="valid.pdf")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def upload_file(self, filename="test_file.pdf", authenticate=True):
        if authenticate:
            self.client.force_authenticate(self.user)
        file = factory.create_simple_uploaded_file(filename=filename)
        response = self.client.post(API_ENDPOINT, {"file": file}, format="multipart")
        return response



from django.conf import settings
from django.test import TestCase

from drf_file_upload import models

FAKE_REQUEST_DOMAIN = "http://drf-file-upload.test"


class FakeRequest:
    def __init__(self, user):
        self.user = user

    def build_absolute_uri(self, url):
        return f"{FAKE_REQUEST_DOMAIN}{url}"


class BaseDrfFileUploadTestCase(TestCase):
    def setUp(self):
        settings.DRF_FILE_UPLOAD_MAX_SIZE = None
        settings.DRF_FILE_UPLOAD_ALLOWED_FORMATS = None

    def tearDown(self):
        models.AnonymousUploadedFile.objects.all().delete()
        models.AuthenticatedUploadedFile.objects.all().delete()



import os
import uuid

from django.conf import settings
from django.db import models
from django.dispatch import receiver


def get_authenticated_uploaded_file_path(instance, filename):
    ext = filename.split(".")[-1]

    # TODO: A setting for the file name too?
    filename = "%s.%s" % (uuid.uuid4(), ext)

    # TODO: Create a setting
    return os.path.join("files/", filename)


def get_anonymous_uploaded_file_path(instance, filename):
    ext = filename.split(".")[-1]

    # TODO: A setting for the file name too?
    filename = "%s.%s" % (uuid.uuid4(), ext)

    # TODO: Create a setting
    return os.path.join("uploads/", filename)


def generate_uuid():
    return str(uuid.uuid4())


class AuthenticatedUploadedFile(models.Model):
    file = models.FileField(blank=False, null=False, upload_to=get_authenticated_uploaded_file_path)
    created = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    uuid = models.CharField(unique=True, max_length=64, default=None, null=True)

    def __str__(self):
        return self.file.name

    def save(self, **kwargs):
        if self.id is None:
            self.uuid = generate_uuid()
        super().save(**kwargs)


class AnonymousUploadedFile(models.Model):
    file = models.FileField(blank=False, null=False, upload_to=get_anonymous_uploaded_file_path)
    uuid = models.CharField(unique=True, max_length=64)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.file.name

    def save(self, **kwargs):
        if self.id is None:
            self.uuid = generate_uuid()
        super().save(**kwargs)


def delete_file_if_exists(instance):
    if instance.file and os.path.isfile(instance.file.path):
        os.remove(instance.file.path)


def delete_file_on_change(model, instance):
    if not instance.pk:
        return False

    try:
        old_file = model.objects.get(pk=instance.pk).file
    except model.DoesNotExist:
        return False

    new_file = instance.file
    if old_file and old_file != new_file:
        if os.path.isfile(old_file.path):
            os.remove(old_file.path)


@receiver(models.signals.post_delete, sender=AuthenticatedUploadedFile)
def delete_files_on_auth_uploaded_file_delete(sender, instance, **kwargs):
    delete_file_if_exists(instance)


@receiver(models.signals.pre_save, sender=AuthenticatedUploadedFile)
def delete_files_on_auth_uploaded_file_change(sender, instance, **kwargs):
    delete_file_on_change(AuthenticatedUploadedFile, instance)


@receiver(models.signals.post_delete, sender=AnonymousUploadedFile)
def delete_files_on_anon_uploaded_file_delete(sender, instance, **kwargs):
    delete_file_if_exists(instance)


@receiver(models.signals.pre_save, sender=AnonymousUploadedFile)
def delete_files_on_anon_uploaded_file_change(sender, instance, **kwargs):
    delete_file_on_change(AnonymousUploadedFile, instance)



from rest_framework import mixins
from rest_framework.parsers import FileUploadParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import GenericViewSet

from drf_file_upload import serializers, models


class AuthenticatedFileUploadView(mixins.CreateModelMixin, mixins.DestroyModelMixin, GenericViewSet):
    parser_class = (FileUploadParser,)
    permission_classes = [IsAuthenticated]
    serializer_class = serializers.AuthenticatedUploadFileSerializer

    def get_queryset(self):
        return models.AuthenticatedUploadedFile.objects.filter(user_id=self.request.user.id)


class AnonymousFileUploadView(mixins.CreateModelMixin, mixins.DestroyModelMixin, GenericViewSet):
    parser_class = (FileUploadParser,)
    permission_classes = []
    serializer_class = serializers.AnonymousUploadFileSerializer

    def get_queryset(self):
        return models.AnonymousUploadedFile.objects.all()



from django.conf import settings
from django.db import models


class TestUserFileUpload(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    file = models.FileField()
    created = models.DateTimeField(auto_now_add=True)


class TestAnonFileUpload(models.Model):

    file = models.FileField()
    created = models.DateTimeField(auto_now_add=True)
